with Util.Times;
with Util.Times.IO;
with Util.Calendars.Western;
with Util.Calendars.Gregorian;
with Util.Calendars.Julian;
with Util.Calendars.Universal;

with Ada.Calendar;
with Util.Calendar;

with Ada.Text_IO;

procedure Calendar_Test is

   use Util.Times;
   use Util.Times.IO;
   use Util.Calendars.Gregorian;
   use Util.Calendars.Julian;
   use Util.Calendars.Universal;

   use Ada.Text_IO;

   G : Util.Calendars.Gregorian.Calendar;
   J : Util.Calendars.Julian.Calendar;
   U : Util.Calendars.Universal.Calendar;

   T : Time;

begin
   T := Time_Of (G, 1582, 10, 15);
   Put_Line
     ("Gregorian 15-OCT-1582 > " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (G, T)));
   T := Time_Of (J, 1582, 10, 4);
   Put_Line
     ("Julian 4-OCT-1582 > " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (J, T)));
   T := Time_Of (J, 1582, 10, 5);
   Put_Line
     ("Julian 5-OCT-1582 > " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (J, T)));
   T := Now;
   Put_Line
     ("Today is a " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   T := Time_Of (U, 1582, 10, 4);
   Put_Line
     ("Universal 4-OCT-1582 > " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   T := Time_Of (U, 1582, 10, 15);
   Put_Line
     ("Universal 15-OCT-1582 > " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   for I in 5 .. 14 loop
      begin
         T := Time_Of (U, 1582, 10, I);
         Put_Line ("Universal " &  Natural'Image (I) &
                   "-OCT-1582 SHOULD FAIL!");
      exception
         when Util.Calendars.Date_Error =>
            Put_Line ("Universal " &  Natural'Image (I) &
                      "-OCT-1582 does not exist.");
      end;
   end loop;
   T := Easter (U, Year (U, Now));
   Put_Line ("Easter this year: " &
             Image (U, T) &
             " (a " &
             Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)) &
             ')');
   T := Easter (U, Year (U, Now) + 1);
   Put_Line ("Easter next year: " &
             Image (U, T) &
             " (a " &
             Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)) &
             ')');
   T := Time_Of (U, 1582, 355);
   Put_Line ("The 355th day of 1582 was" &
             Image (U, T) &
             " (a " &
             Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)) &
             ')');
   T := Time_Of (U, 1583, 1, 1);
   Put_Line ("Universal 1-JAN-1583 was a " &
             Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   for I in 275 .. 281 loop
      T := Time_Of (U, 1582, I);
      Put_Line ("The" & Natural'Image (I) & "th day of 1582 was " &
                Image (U, T) &
                " (a " &
                Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)) &
                ')');
   end loop;
   T := Time_Of (U, 2002, 06, 19);
   Put_Line
     ("Universal 19-JUN-2002 (Wed) > " &
         Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   T := Time_Of (U, 1999, 06, 19);
   Put_Line
     ("Universal 19-JUN-1999 (Sat) > " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   T := Time_Of (U, 2002, 02, 03);
   Put_Line
     ("Universal 03-FEB-2002 (Sun) > " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   T := Time_Of (U, 2000, 03, 03);
   Put_Line
     ("Universal 03-MAR-2000 was a " &
      Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   T := Time_Of (U, 1965, 4, 15);
   Put_Line ("Universal 15-APR-1965 was a " &
             Util.Calendars.Western.Week_Day'Image (Day_Of_Week (U, T)));
   T := Sub_Months (U, T, 3);
   Put_Line ("Three months earlier is " & Image (U, T));
   T := Sub_Months (U, T, 3);
   Put_Line ("Another three months earlier is " & Image (U, T));
   T := Add_Months (U, T, 2);
   Put_Line ("Two months later than that is " & Image (U, T));
   T := Add_Months (U, T, 2);
   Put_Line ("Another two months later than that is " & Image (U, T));
   T := Add_Months (U, T, 2);
   Put_Line ("And yet another two months later than that is " & Image (U, T));
   T := Add_Months (U, T, 12);
   Put_Line ("Twelve months later is " &  Image (U, T));
   T := Sub_Months (U, T, 12);
   Put_Line ("Twelve months earlier than that is " &  Image (U, T));
   T := Add_Months (U, T, 15);
   Put_Line ("15 months later is " &  Image (U, T));
   T := Sub_Months (U, T, 15);
   Put_Line ("15 months earlier than that is " &  Image (U, T));
   T := Time_Of (U, 2001, 10, 1, 3_600.0);
   declare
      T2 : Time;
      D  : Time_Difference;
   begin
      D := To_Duration (0, 7_200.0);
      T2 := T - D;
      Put_Line ("1-10-2001 01:00:00 - two hours : " &
                Image (U, T2) & ' ' &  Image (Secs (T2)));
      D := To_Duration (1, 7_200.0);
      T2 := T - D;
      Put_Line ("1-10-2001 01:00:00 - 1 day and two hours : " &
                Image (U, T2) & ' ' &  Image (Secs (T2)));
      T2 := T2 + D;
      Put_Line ("And + 1 day and 2 hours again : " &
                Image (U, T2) & ' ' & Image (Secs (T2)));
      declare
         D2 : Time_Difference := To_Duration (1, 86_400.0 - 7_200.0);
      begin
         T2 := T + (D2 + D);
         Put_Line ("And 3 days later: " &
                   Image (U, T2) & ' ' & Image (Secs (T2)));
      end;
      declare
         D3 : Time_Difference := To_Duration (0, 86_400.0 / 2);
      begin
         T2 := T + 6 * D3;
         Put_Line ("And 3 days later again: " &
                   Image (U, T2) & ' ' & Image (Secs (T2)));
      end;
      declare
         D3 : Time_Difference := To_Duration (0, 86_400.0);
      begin
         T2 := T + 3.5 * D3;
         Put_Line ("And 3.5 days later again: " &
                   Image (U, T2) & ' ' & Image (Secs (T2)));
      end;
      declare
         T : Day_Duration;
      begin
         T := 86_400.0 - 0.335;
         Put_Line ("23:59:59.665 : " & Image (T, False, False));
         Put_Line ("23:59:59.665 : " & Image (T, False, True));
         Put_Line ("23:59:59.665 : " & Image (T, True, False));
         Put_Line ("23:59:59.665 : " & Image (T, True, True));
         Put_Line ("23:59:59.665 : " & Image (T, 0, False));
         Put_Line ("23:59:59.665 : " & Image (T, 1, False));
         Put_Line ("23:59:59.665 : " & Image (T, 2, False));
         Put_Line ("23:59:59.665 : " & Image (T, 3, False));
         Put_Line ("23:59:59.665 : " & Image (T, 4, False));
         Put_Line ("23:59:59.665 : " & Image (T, 0, True));
         Put_Line ("23:59:59.665 : " & Image (T, 1, True));
         Put_Line ("23:59:59.665 : " & Image (T, 2, True));
         Put_Line ("23:59:59.665 : " & Image (T, 3, True));
         Put_Line ("23:59:59.665 : " & Image (T, 4, True));
         T := 86_400.0 - 0.501;
         Put_Line ("23:59:59.499 : " & Image (T, 0, False));
         Put_Line ("23:59:59.499 : " & Image (T, 1, False));
         Put_Line ("23:59:59.499 : " & Image (T, 2, False));
         Put_Line ("23:59:59.499 : " & Image (T, 3, False));
         Put_Line ("23:59:59.499 : " & Image (T, 4, False));
         Put_Line ("23:59:59.499 : " & Image (T, 0, True));
         Put_Line ("23:59:59.499 : " & Image (T, 1, True));
         Put_Line ("23:59:59.499 : " & Image (T, 2, True));
         Put_Line ("23:59:59.499 : " & Image (T, 3, True));
         Put_Line ("23:59:59.499 : " & Image (T, 4, True));
      end;
   end;
   declare
      use Ada.Calendar;
      use Util.Calendar;
   begin
      Put_Line ("Std Today > " &
                Weekday'Image (Day_Of_Week (Clock)));
      Put_Line ("Std 19-JUN-2002 (Wed) > " &
                Weekday'Image (Day_Of_Week (Time_Of (2002, 06, 19))));
      Put_Line ("Std 19-JUN-1999 (Sat) > " &
                Weekday'Image (Day_Of_Week (Time_Of (1999, 06, 19))));
      Put_Line ("Std 03-FEB-2002 (Sun) > " &
                Weekday'Image (Day_Of_Week (Time_Of (2002, 02, 03))));
      Put_Line ("Std 15-APR-1965 (Thu) > " &
                Weekday'Image (Day_Of_Week (Time_Of (1965, 04, 15))));
      Put_Line ("Std 03-MAR-2000 > " &
                Weekday'Image (Day_Of_Week (Time_Of (2000, 03, 03))));
   end;
end Calendar_Test;


