with Util.Strings;
with Ada.Text_IO;
with Ada.Strings.Unbounded;

procedure Test_String is

   use Util.Strings;
   use Ada.Text_IO;
   use Ada.Strings.Unbounded;

   type Test is
      record
         S    : Unbounded_String;
         I, J : Natural;
         E    : Character;
      end record;

   type Test_Table is array (Positive range <>) of Test;

   Tests : Test_Table :=
     ((To_Unbounded_String (""), 0, 0, No_Escape),
      (To_Unbounded_String ("This is ""a string"", isn't it?"), 9, 18,
       No_Escape),
      (To_Unbounded_String ("""a string"""), 1, 10, No_Escape),
      (To_Unbounded_String ("This is ""a string"), 9, 0, No_Escape),
      (To_Unbounded_String (""), 0, 0, '\'),
      (To_Unbounded_String ("This is ""a string"", isn't it?"), 9, 18, '\'),
      (To_Unbounded_String ("""a string"""), 1, 10, '\'),
      (To_Unbounded_String ("This is ""a string"), 9, 0, '\'),
      (To_Unbounded_String ("""a string with \\ and \"""""), 1, 25, '\'),
      (To_Unbounded_String ("""a string with \\ and \\"""""), 1, 25, '\'),
      (To_Unbounded_String ("""a string with \\ and \\\"""""), 1, 27, '\'),
      (To_Unbounded_String ("""a string with \\ and \\\"""), 1, 0, '\'),
      (To_Unbounded_String ("""a string with \\ and \\"""), 1, 25, '\'),
      (To_Unbounded_String ("""a string with \\ and \\"), 1, 0, '\'),
      (To_Unbounded_String ("""a string with \\ and \"""), 1, 0, '\'),
      (To_Unbounded_String ("""a string with \\ and \"), 1, 0, '\'),
      (To_Unbounded_String (""), 0, 0, '"'),
      (To_Unbounded_String ("""a string with """" embedded"" "), 1, 27, '"'),
      (To_Unbounded_String ("""a string with """" embedded "), 1, 0, '"'),
      (To_Unbounded_String ("""a string with "" embedded "), 1, 16, '"'),
      (To_Unbounded_String ("""a string with "" "" embedded "), 1, 16, '"'),
      (To_Unbounded_String ("""a string with \"" "" embedded "), 1, 17, '"'),
      (To_Unbounded_String (""""), 1, 0, '"'),
      (To_Unbounded_String (""""""), 1, 2, '"'),
      (To_Unbounded_String (""""""""), 1, 0, '"')
     );

   I          : Natural   := Tests'First;
   From, To   : Natural;
   Nof_Errors : Natural := 0;

begin
   while I <= Tests'Last loop
      Get_String (To_String (Tests (I).S), From, To, '"', Tests (I).E);
      if From /= Tests (I).I or else To /= Tests (I).J then
         Nof_Errors := Nof_Errors + 1;
         Put_Line ("Error: got (" &
                   Natural'Image (From) & ',' &
                   Natural'Image (To) & "), expected (" &
                   Natural'Image (Tests (I).I) & ',' &
                   Natural'Image (Tests (I).J) & ") for string <" &
                   To_String (Tests (I).S) & '>');
         Put ("Escape was ");
         if Tests (I).E = No_Escape then
            Put_Line ("NIL");
         else
            Put_Line (Tests (I).E & "");
         end if;
      end if;
      I := I + 1;
   end loop;
   if Nof_Errors = 0 then
      Put_Line ("ALL TESTS COMPLETED SUCCESSFULLY!");
   else
      Put_Line ("TESTS DONE;" & Natural'Image (Nof_Errors) & " FAILURES!");
   end if;
end Test_String;
