with Util.Calendars.Universal;
with Util.Calendars.Julian;
with Util.Times.IO;
with Ada.Text_IO;

procedure Test_Switch_Date
is
   use Util.Calendars.Universal;
   use Util.Times;
   use Util.Times.IO;
   use Ada.Text_IO;

   package Jul renames Util.Calendars.Julian;

   C : Calendar;
   J : Jul.Calendar;
   T : Time;
   Y : Year_Number;
   M : Month_Number;
   D : Day_Number;

   procedure Test_Switch
     (Y    : in Year_Number;
      M    : in Month_Number;
      From : in Day_Number;
      To   : in Day_Number)
   is
      D : Day_Number := From;
   begin
      while D <= To loop
         begin
            Put_Line (Image (C, Time_Of (C, Y, M, D)) & " exists");
         exception
            when Util.Calendars.Date_Error =>
               Put_Line (Year_Number'Image (Y) & '-' &
                         Month_Number'Image (M) & '-' &
                         Day_Number'Image (D) &  " doesn't exist");
         end;
         exit when D = To;
         D := D + 1;
      end loop;
   end Test_Switch;

begin
   Put_Line ("Today is " & Image (C, Now));
   --  Check the default setting:
   Test_Switch (1582, 10, 3, 16);
   --  Now do the default setting explicitly:
   New_Line;
   Set_Switch_Date (C, Jul.Time_Of (J, 1582, 10, 4));
   Test_Switch (1582, 10, 3, 16);
   --  Now a setting that will cause two years to be affected:
   New_Line;
   Set_Switch_Date (C, Jul.Time_Of (J, 1582, 12, 30));
   Test_Switch (1582, 10, 3, 16);
   Test_Switch (1582, 12, 25, 31);
   Test_Switch (1583, 1, 1, 15);
   --  Now a setting in the next century:
   New_Line;
   Set_Switch_Date (C, Jul.Time_Of (J, 1601, 12, 31));
   Test_Switch (1601, 12, 25, 31);
   Test_Switch (1602, 1, 1, 15);
   --  And a setting in a gregorian non-leap year:
   New_Line;
   Set_Switch_Date (C, Jul.Time_Of (J, 1600, 12, 27));
   Test_Switch (1600, 12, 25, 31);
   Test_Switch (1601, 1, 1, 15);
   --  And again a century later:
   New_Line;
   Set_Switch_Date (C, Jul.Time_Of (J, 1701, 12, 31));
   Test_Switch (1701, 12, 25, 31);
   Test_Switch (1702, 1, 1, 15);
   --  And a setting in a gregorian leap year:
   New_Line;
   Set_Switch_Date (C, Jul.Time_Of (J, 1700, 12, 27));
   Test_Switch (1700, 12, 25, 31);
   Test_Switch (1701, 1, 1, 15);
   --  And in a leap year:
   New_Line;
   Set_Switch_Date (C, Jul.Time_Of (J, 1708, 2, 24));
   Test_Switch (1708, 2, 20, 29);
   Test_Switch (1708, 3, 1, 15);

   Put_Line ("All tests done.");
end Test_Switch_Date;


