with Util.Strings;
with Ada.Text_IO;
with Ada.Exceptions;
with Ada.Strings.Unbounded;

procedure Test_Wildcard is

   use Ada.Strings.Unbounded;

   type Test_String is
      record
         Text    : Unbounded_String;
         Outcome : Boolean;
      end record;

   type Test is
      record
         Pattern : Unbounded_String;
         Against : Test_String;
      end record;

   type Test_Array is array (Natural range <>) of Test;

   Tests : Test_Array :=
     ((To_Unbounded_String ("?*?"),
       (To_Unbounded_String ("a"), False)),

      (To_Unbounded_String ("*ello"),
       (To_Unbounded_String ("helloello"), True)),
      (To_Unbounded_String ("*ello"),
       (To_Unbounded_String ("helloelko"), False)),
      (To_Unbounded_String ("*ello"),
       (To_Unbounded_String ("ello"), True)),
      (To_Unbounded_String ("*ello"),
       (To_Unbounded_String ("hello"), True)),
      (To_Unbounded_String ("*ello"),
       (To_Unbounded_String ("helko"), False)),

      (To_Unbounded_String ("*"),
       (To_Unbounded_String (""), True)),
      (To_Unbounded_String ("*"),
       (To_Unbounded_String ("sDJGJDHGHDGHJDHSSSSSSShjhfddddhdsjitriueujsfbjnnvfddghreguhreeegaggghtiu"), True)),

      (To_Unbounded_String (""),
       (To_Unbounded_String (""), True)),
      (To_Unbounded_String (""),
       (To_Unbounded_String ("a"), False)),
      (To_Unbounded_String (""),
       (To_Unbounded_String ("hello"), False)),

      (To_Unbounded_String ("\?"),
       (To_Unbounded_String ("?"), True)),
      (To_Unbounded_String ("\?"),
       (To_Unbounded_String ("?x"), False)),

      (To_Unbounded_String ("\??"),
       (To_Unbounded_String ("?x"), True)),
      (To_Unbounded_String ("\??"),
       (To_Unbounded_String ("?"), False)),

      (To_Unbounded_String ("\?*"),
       (To_Unbounded_String ("?"), True)),
      (To_Unbounded_String ("\?*"),
       (To_Unbounded_String ("??"), True)),

      (To_Unbounded_String ("*.*"),
       (To_Unbounded_String ("hello.world"), True)),
      (To_Unbounded_String ("*.*"),
       (To_Unbounded_String ("hello.world.nice.day"), True)),
      (To_Unbounded_String ("*.*"),
       (To_Unbounded_String ("hello"), False)),

      (To_Unbounded_String ("-*-*-*-*-*-*-12-*-*-*-m-*-*-*"),
       (To_Unbounded_String ("-adobe-courier-bold-o-normal--12-120-75-75-m-70-iso8859-1"), True)),
      (To_Unbounded_String ("-*-*-*-*-*-*-12-*-*-*-m-*-*-*"),
       (To_Unbounded_String ("-adobe-courier-bold-o-normal--12-120-75-75-X-70-iso8859-1"), False)),

      (To_Unbounded_String ("*a*b*c*d"),
       (To_Unbounded_String ("abcxxxxxxxxxxxxxxxxxxxxxxxxxxxx"), False)),
      (To_Unbounded_String ("*a*b*c*d"),
       (To_Unbounded_String ("abcxxxxxxxxxxxxxxxxxxxxxxxxxxxxd"), True)),
      (To_Unbounded_String ("*a*b*c*d"),
       (To_Unbounded_String ("abcxxxxxxxxxxxxxxxxxxxxxxxxxxxxdxxx"), False)),

      (To_Unbounded_String ("*a*b*c*d*"),
       (To_Unbounded_String ("abcxxxxxxxxxxxxxxxxxxxxxxxxxxxx"), False)),
      (To_Unbounded_String ("*a*b*c*d*"),
       (To_Unbounded_String ("abcxxxxxxxxxxxxxxxxxxxxxxxxxxxxd"), True)),
      (To_Unbounded_String ("*a*b*c*d*"),
       (To_Unbounded_String ("abcxxxxxxxxxxxxxxxxxxxxxxxxxxxxdxxx"), True)),

      (To_Unbounded_String ("*.c"),
       (To_Unbounded_String ("hello.c~"), False)),
      (To_Unbounded_String ("*.c"),
       (To_Unbounded_String ("hello.c"), True)),
      (To_Unbounded_String ("*.c"),
       (To_Unbounded_String (".c"), True)),
      (To_Unbounded_String ("*.c"),
       (To_Unbounded_String (".cshrc"), False)),

      (To_Unbounded_String ("*.c*"),
       (To_Unbounded_String ("hello.c~"), True)),
      (To_Unbounded_String ("*.c*"),
       (To_Unbounded_String ("hello.c"), True)),
      (To_Unbounded_String ("*.c*"),
       (To_Unbounded_String (".c"), True)),
      (To_Unbounded_String ("*.c*"),
       (To_Unbounded_String (".cshrc"), True)),

      (To_Unbounded_String ("a?"),
       (To_Unbounded_String ("a"), False)),
      (To_Unbounded_String ("a?"),
       (To_Unbounded_String ("aa"), True)),
      (To_Unbounded_String ("a?"),
       (To_Unbounded_String ("aaa"), False)),

      (To_Unbounded_String ("abc[de]f"),
       (To_Unbounded_String ("abc"), False)),
      (To_Unbounded_String ("abc[de]f"),
       (To_Unbounded_String ("abcf"), False)),
      (To_Unbounded_String ("abc[de]f"),
       (To_Unbounded_String ("abcdf"), True)),
      (To_Unbounded_String ("abc[de]f"),
       (To_Unbounded_String ("abcdef"), False)),
      (To_Unbounded_String ("abc[de]f"),
       (To_Unbounded_String ("abcef"), True)),

      (To_Unbounded_String ("a[q-t]b*c"),
       (To_Unbounded_String ("abc"), False)),
      (To_Unbounded_String ("a[q-t]b*c"),
       (To_Unbounded_String ("abdc"), False)),
      (To_Unbounded_String ("a[q-t]b*c"),
       (To_Unbounded_String ("aqbc"), True)),
      (To_Unbounded_String ("a[q-t]b*c"),
       (To_Unbounded_String ("a-bc"), False)),
      (To_Unbounded_String ("a[q-t]b*c"),
       (To_Unbounded_String ("atbqbc"), True)),
      (To_Unbounded_String ("a[q-t]b*c"),
       (To_Unbounded_String ("asbwc"), True)),
      (To_Unbounded_String ("a[q-t]b*c"),
       (To_Unbounded_String ("aubec"), False)),

      (To_Unbounded_String ("a[!d-f]b"),
       (To_Unbounded_String ("acb"), True)),
      (To_Unbounded_String ("a[!d-f]b"),
       (To_Unbounded_String ("adb"), False)),
      (To_Unbounded_String ("a[!d-f]b"),
       (To_Unbounded_String ("aeb"), False)),
      (To_Unbounded_String ("a[!d-f]b"),
       (To_Unbounded_String ("afb"), False)),
      (To_Unbounded_String ("a[!d-f]b"),
       (To_Unbounded_String ("a[b"), True)),
      (To_Unbounded_String ("a[!d-f]b"),
       (To_Unbounded_String ("agb"), True)),

      (To_Unbounded_String ("a*[a-cx]b"),
       (To_Unbounded_String ("acb"), True)),
      (To_Unbounded_String ("a*[a-cx]b"),
       (To_Unbounded_String ("ab"), False)),
      (To_Unbounded_String ("a*[a-cx]b"),
       (To_Unbounded_String ("axb"), True)),
      (To_Unbounded_String ("a*[a-cx]b"),
       (To_Unbounded_String ("adb"), False)),
      (To_Unbounded_String ("a*[a-cx]b"),
       (To_Unbounded_String ("a-b"), False)),
      (To_Unbounded_String ("a*[a-cx]b"),
       (To_Unbounded_String ("axxxxxxxxxxxxxdcb"), True)),
      (To_Unbounded_String ("a*[a-cx]b"),
       (To_Unbounded_String ("axxxxxxxxxxxxxdb"), False)),
      (To_Unbounded_String ("a*[a-cx]b"),
       (To_Unbounded_String ("axxxxxxxxxxxxxb"), True)),

      (To_Unbounded_String ("a[!!?]b"),
       (To_Unbounded_String ("acb"), True)),
      (To_Unbounded_String ("a[!!?]b"),
       (To_Unbounded_String ("a!b"), False)),
      (To_Unbounded_String ("a[!!?]b"),
       (To_Unbounded_String ("a?b"), False)),

      (To_Unbounded_String ("a[][]b"),
       (To_Unbounded_String ("acb"), False)),
      (To_Unbounded_String ("a[][]b"),
       (To_Unbounded_String ("a]b"), True)),
      (To_Unbounded_String ("a[][]b"),
       (To_Unbounded_String ("a[b"), True)),

      (To_Unbounded_String ("a[!][]b"),
       (To_Unbounded_String ("acb"), True)),
      (To_Unbounded_String ("a[!][]b"),
       (To_Unbounded_String ("a]b"), False)),
      (To_Unbounded_String ("a[!][]b"),
       (To_Unbounded_String ("a[b"), False)),
      (To_Unbounded_String ("a[!][]b"),
       (To_Unbounded_String ("a!b"), True)),

      (To_Unbounded_String ("a[!][!]b"),
       (To_Unbounded_String ("acb"), True)),
      (To_Unbounded_String ("a[!][!]b"),
       (To_Unbounded_String ("a]b"), False)),
      (To_Unbounded_String ("a[!][!]b"),
       (To_Unbounded_String ("a[b"), False)),
      (To_Unbounded_String ("a[!][!]b"),
       (To_Unbounded_String ("a!b"), False))

      );

   Zero_Tests : Test_Array :=
     ((To_Unbounded_String ("a?b"),
       (To_Unbounded_String ("ab"), True)),
      (To_Unbounded_String ("a?b"),
       (To_Unbounded_String ("acb"), True)),
      (To_Unbounded_String ("a?b"),
       (To_Unbounded_String ("acdb"), False)),

      (To_Unbounded_String ("?ab"),
       (To_Unbounded_String ("ab"), True)),
      (To_Unbounded_String ("?ab"),
       (To_Unbounded_String ("cab"), True)),
      (To_Unbounded_String ("?ab"),
       (To_Unbounded_String ("abb"), False)),

      (To_Unbounded_String ("ab?"),
       (To_Unbounded_String ("ab"), True)),
      (To_Unbounded_String ("ab?"),
       (To_Unbounded_String ("abc"), True)),
      (To_Unbounded_String ("ab?"),
       (To_Unbounded_String ("abb"), True)),
      (To_Unbounded_String ("ab?"),
       (To_Unbounded_String ("abbd"), False)),

      (To_Unbounded_String ("a?*b"),
       (To_Unbounded_String ("ab"), True)),
      (To_Unbounded_String ("a?*b"),
       (To_Unbounded_String ("acb"), True)),
      (To_Unbounded_String ("a?*b"),
       (To_Unbounded_String ("acdb"), True)),
      (To_Unbounded_String ("a?*b"),
       (To_Unbounded_String ("acdbc"), False))

      );

   protected Counter is
      procedure Inc;
      function Get return Natural;
   private
      N : Natural := 0;
   end Counter;

   protected body Counter is
      procedure Inc is
      begin
         N := N + 1;
      end Inc;
      function Get return Natural is
      begin
         return N;
      end Get;
   end Counter;

   use Ada.Text_IO;
   use Ada.Exceptions;

   function Match_Zero is
      new Util.Strings.Wildcard_Match (Zero_Or_One => True);

   procedure Test_Them (Tests : in Test_Array; Zero_OK : in Boolean)
   is
   begin
      for I in Tests'Range loop
         Put_Line ("Pattern:  " & To_String (Tests (I).Pattern));
         Put_Line ("Text:     " & To_String (Tests (I).Against.Text));
         Put_Line ("Expected: " & Boolean'Image (Tests (I).Against.Outcome));
         declare
            Outcome : Boolean;
            use Util.Strings;
         begin
            select
               delay 15.0;
               Put_Line (">>> TIMEOUT AFTER 15 SECONDS!!");
               Counter.Inc;
            then abort
               if Zero_OK then
                  Outcome :=
                     Match_Zero (Pattern => To_String (Tests (I).Pattern),
                                 Text    => To_String (Tests (I).Against.Text));
               else
                  Outcome :=
                     Match (Pattern => To_String (Tests (I).Pattern),
                            Text    => To_String (Tests (I).Against.Text));
               end if;
               if Outcome = Tests (I).Against.Outcome then
                  Put_Line (">>> Success!");
               else
                  Put_Line (">>> FAILED!!!"); Counter.Inc;
               end if;
            end select;
         exception
            when E : others =>
               Put_Line (">>> RAISED " & Exception_Name (E));
               Put_Line (">>> Info = " & Exception_Information (E));
               Counter.Inc;
         end;
      end loop;
   end Test_Them;

begin
   Put_Line ("Wildcard matching test...");
   Test_Them (Tests, False);
   Test_Them (Zero_Tests, True);
   declare
      N : Natural := Counter.Get;
   begin
      if N = 0 then
         Put_Line ("*** ALL" &
                   Natural'Image (Tests'Length + Zero_Tests'Length) &
                   " TESTS SUCCEEDED!");
      else
         Put_Line ("***" & Natural'Image (N) & "TEST(S) OF" &
                   Natural'Image (Tests'Length +  Zero_Tests'Length) &
                   " FAILED!");
      end if;
   end;
end Test_Wildcard;





