-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2001, 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <VERSION ID="1.1">
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    Root package for the calendar subsystem.
--  </PURPOSE>
--
--  <NOT_TASK_SAFE>
--
--  <NO_STORAGE>
--
--  <HISTORY>
--    03-OCT-2001   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

with Util.Times;

package Util.Calendars is

   Date_Error : exception;
   --  Any calendar operation may raise this exception if a date is not valid
   --  for a particular calendar. Possible reasons for @Date_Error@ being
   --  raised include: passing a date before 15-OCT-1582 to a Gregorian
   --  calendar, or passing a non-existing date to a Universal calendar, such
   --  as a date in the range 05-OCT-1582 to 14-OCT-1582, if the switch date
   --  of the Universal calendar is not changed.

   type Calendar is abstract tagged null record;
   --  Abstract root type for all calendars. Note: any other operations are
   --  added by derived types. For instance, operations on days and months
   --  are added in child package <CODE>
   --  <A HREF="util-calendars-western.html">Util.Calendars.Western</A></CODE>.
   --
   --  They're not added here because other calendar systems may not have
   --  a concept of months in the sense of the western calendars.

   subtype Year_Number is Integer;
   --  But any calendar has a concept of a year! (Even if the counting systems
   --  may differ wildly, as may the length of a year.)

   function Year
     (Base : in Calendar;
      Date : in Util.Times.Time)
     return Year_Number
     is abstract;
   --  Return the year of the @Date@. May raise @Date_Error@ if the @Date@ is
   --  not a valid date for the given calendar @Base@.

   function Image
     (Base : in Calendar;
      Date : in Util.Times.Time)
     return String
      is abstract;
   --  Returns a string representation of the given @Date@. May raise
   --  @Date_Error@ if the @Date@ is not a valid date for the given calendar
   --  @Base@.

end Util.Calendars;
