-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2001, 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    Bash-style variable substitution.
--  </PURPOSE>
--
--  <NOT_TASK_SAFE>
--
--  <STORAGE>
--    No dynamic storage allocation. @Expand@ is recursive; the recursion
--    depth (and hence the stack consumption) is limited by the number of
--    variable references and escaped '@$@'-signs in the @Source@ string.
--  </STORAGE>
--
--  <HISTORY>
--    03-MAY-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

with Util.Strings;

package body Util.Environment.Windows is

   function Expand
     (Self   : access Windows_Expander;
      Source : in     String)
     return String
   is
      use Util.Strings;

      Start, Stop, From : Natural;
   begin
      From := Source'First;
      while From < Source'Last loop
         Get_String (Source (From .. Source'Last), Start, Stop, '%');
         exit when Stop = 0; --  No '%' found, or no closing '%' found.
         if Stop = Start + 1 then
            --  "%%" found: literal % sign.
            return Source (Source'First .. Start) &
                   Expand (Windows_Expander'Class (Self.all)'Access,
                           Source (Stop + 1 .. Source'Last));
         elsif Is_In (Blanks, Source (Start + 1)) then
            --  '%' followed by white space.
            From := Stop;
         else
            return Source (Source'First .. Start - 1) &
                   Get (Windows_Expander'Class (Self.all)'Access,
                        Source (Start + 1 .. Stop - 1)) &
                   Expand (Windows_Expander'Class (Self.all)'Access,
                           Source (Stop + 1 .. Source'Last));
         end if;
      end loop;
      --  No reference found.
      return Source;
   end Expand;

end Util.Environment.Windows;
