-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2001, 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    Floor, Ceiling, Truncation and Fraction for arbitrary fixed-point values.
--    Any of these may raise @Constraint_Error@ if the result isn't in the
--    range of type @Fixed_Point@.
--  </PURPOSE>
--
--  <TASKING>
--    Fully task-safe.
--  </TASKING>
--
--  <NO_STORAGE>
--
--  <HISTORY>
--    28-OCT-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

with System;

--  generic
--     type Fixed_Point is delta <>;
package body Util.Fixed is

   type Largest_Int is range System.Min_Int .. System.Max_Int;

   function Floor
     (X : in Fixed_Point)
     return Fixed_Point
   is
      Tmp : Largest_Int := Largest_Int (X);
   begin
      if Fixed_Point (Tmp) > X then Tmp := Tmp - 1; end if;
      return Fixed_Point (Tmp);
   end Floor;

   function Ceiling
     (X : in Fixed_Point)
     return Fixed_Point
   is
      Tmp : Largest_Int := Largest_Int (X);
   begin
      if Fixed_Point (Tmp) < X then Tmp := Tmp + 1; end if;
      return Fixed_Point (Tmp);
   end Ceiling;

   function Fraction
     (X : in Fixed_Point)
     return Fixed_Point
   is
   begin
      return X - Truncated (X);
   end Fraction;

   function Truncated
     (X : in Fixed_Point)
     return Fixed_Point
   is
   begin
      if X >= 0.0 then
         return Floor (X);
      else
         return Ceiling (X);
      end if;
   end Truncated;

end Util.Fixed;

