-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2001, 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <VERSION ID="1.1">
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    A generic package that, when instantiated anywhere, provides the name
--    of the instantiating unit. This name is implementation-defined if the
--    package is instantiated within an unnamed block statement.
--  </PURPOSE>
--
--  <NOT_TASK_SAFE>
--
--  <NO_STORAGE>
--
--  <HISTORY>
--    01-MAR-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

with Ada.Tags;
with Util.Strings;

generic
package Util.Reflect is

   Unit_Name : constant String;
   --  The name of the place where this unit is instantiated.

private

   type Dummy is abstract tagged null record;

   Name  : String  := Ada.Tags.Expanded_Name (Dummy'Tag);

   use Util.Strings;

   Unit_Name : constant String :=
     To_Mixed
       (Name (Name'First ..
              Natural'Max (Index (Name (Name'First ..
                                        Index (Name, '.', Backward)),
                                  '.', Backward), 1) - 1));
   --  Go back two periods: the name is something like "X.Y.Z.Package.Dummy",
   --  and we want to get "X.Y.Z". The whole business about Natural'Max and
   --  subtracting oneq is to avoid a neagtive value if we didn't find two
   --  periods after all.

end Util.Reflect;
