-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2001, 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    Stop watches; useful for simple timing tasks.
--  </PURPOSE>
--
--  <NOT_TASK_SAFE>
--
--  <NO_STORAGE>
--
--  <HISTORY>
--    28-FEB-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

package body Util.Stop_Watches is

   use Ada.Calendar;

   procedure Reset
     (Watch : in out Stop_Watch)
   is
   begin
      Watch.Nof_Starts := 0;
      Watch.Running    := False;
      Watch.Elapsed    := 0.0;
   end Reset;

   procedure Start
     (Watch : in out Stop_Watch)
   is
   begin
      if Watch.Running then raise Stop_Watch_Error; end if;
      Watch.Nof_Starts := Watch.Nof_Starts + 1;
      Watch.Running    := True;
      Watch.Last_Start := Clock;
   end Start;

   procedure Stop
     (Watch : in out Stop_Watch)
   is
   begin
      if Watch.Running then
         Watch.Elapsed := Watch.Elapsed + (Clock - Watch.Last_Start);
         Watch.Running := False;
      end if;
   end Stop;

   function Elapsed
     (Watch : in Stop_Watch)
     return Duration
   is
   begin
      if Watch.Running then raise Stop_Watch_Error; end if;
      return Watch.Elapsed;
   end Elapsed;

   function Nof_Timings
     (Watch : in Stop_Watch)
     return Natural
   is
   begin
      return Watch.Nof_Starts;
   end Nof_Timings;

   function Average
     (Watch : in Stop_Watch)
     return Duration
   is
   begin
      if Watch.Nof_Starts = 0 or else Watch.Running then
         raise Stop_Watch_Error;
      end if;
      return Watch.Elapsed / Watch.Nof_Starts;
   end Average;

end Util.Stop_Watches;
