-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <VERSION ID="1.1">
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    Provides a lock promotion conflict resolution policy that aborts the
--    request of the task with the lower priority.
--
--    Provided here instead of in
--    <CODE><A HREF="util-tasking-complex_locks.html">Complex_Locks</A></CODE>
--    because the implementation depends on package @Ada.Dynamic_Priorities@,
--    which is defined in the "Real Time Systems" annex, not in the "Systems
--    Programming" annex. (There might be Ada implementations that support
--    annex C, but not annex D. Separating out the stuff that needs annex D
--    facilitates thus porting this library to implementations without annex D
--    support: just remove this package.)
--  </PURPOSE>
--
--  <NO_STORAGE>
--
--  <TASKING>
--    Fully task-safe.
--  </TASKING>
--
--  <HISTORY>
--    10-OCT-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

with Ada.Task_Identification;

with Util.Tasking.Complex_Locks;

package Util.Tasking.Conflict_Resolvers is

   pragma Elaborate_Body;

   type Priority_Conflict_Resolver is
     new Util.Tasking.Complex_Locks.Conflict_Resolver with null record;

   function Decide
     (Self        : access Priority_Conflict_Resolver;
      Queued_Task : in     Ada.Task_Identification.Task_Id;
      New_Task    : in     Ada.Task_Identification.Task_Id)
     return Boolean;
   --  Returns @True@ if the priority of the task denoted by @New_Task@ is
   --  @<=@ the priority of the task denoted by @Queued_Task@.

end Util.Tasking.Conflict_Resolvers;
