-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    Controlled types for using locks from package
--    <CODE><A HREF="util-tasking-locks.html">Util.Tasking.Locks</A></CODE>
--    in an exception- and ATC-safe way.
--  </PURPOSE>
--
--  <NO_STORAGE>
--
--  <TASKING>
--    Fully task- and abortion-safe; but see the comments in package
--    <CODE><A HREF="util-tasking-locks.html">Util.Tasking.Locks</A></CODE>
--    about deserters.
--  </TASKING>
--
--  <HISTORY>
--    10-OCT-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

with Ada.Task_Identification;

package body Util.Tasking.Protectors is

   procedure Initialize (Self : in out Lock_Protector)
   is
   begin
      Self.Lock.Acquire;
   end Initialize;

   procedure Finalize   (Self : in out Lock_Protector)
   is
   begin
      Self.Lock.Release;
   end Finalize;

   ----------------------------------------------------------------------------

   procedure Initialize (Self : in out Read_Write_Protector)
   is
   begin
      Self.Lock.Acquire (Self.Read_Write);
   end Initialize;

   procedure Finalize   (Self : in out Read_Write_Protector)
   is
   begin
      Self.Lock.Release;
   end Finalize;

   ----------------------------------------------------------------------------

   procedure Initialize (Self : in out Task_Lock_Protector)
   is
   begin
      Self.Lock.Acquire;
   end Initialize;

   procedure Finalize   (Self : in out Task_Lock_Protector)
   is
      use Ada.Task_Identification;
   begin
      if Self.Lock.Holder = Current_Task then
         Self.Lock.Release;
      end if;
   exception
      when Tasking_Error =>
         --  Ooops, something went really haywire.
         null;
   end Finalize;

end Util.Tasking.Protectors;
