-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <VERSION ID="1.1">
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    Root package of the tasking utilities subsystem. Provides two generic
--    packages for making arbitrary (constrained, non-limited) types task-safe.
--  </PURPOSE>
--
--  <NO_STORAGE>
--
--  <TASKING>
--    Fully task- and abortion-safe.
--  </TASKING>
--
--  <HISTORY>
--    10-OCT-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

package Util.Tasking is

   pragma Elaborate_Body;

   ----------------------------------------------------------------------------

   generic
      type Item is private;
      Default : in Item;
   package Protect is

      protected type Protected_Item is
         procedure Set (To : in Item);
         function  Get return Item;
         procedure Set_And_Get
           (New_Value : in     Item;
            Old_Value :    out Item);
      private
         Value : Item := Default;
      end Protected_Item;

   end Protect;

   ----------------------------------------------------------------------------

   generic
      type Item is private;
      Default : in Item;
      with function "=" (Left, Right : in Item) return Boolean is <>;
   package Protect_With_Wait is

      protected type Protected_Item is
         procedure Set (To : in Item);
         function  Get return Item;
         procedure Set_And_Get
           (New_Value : in     Item;
            Old_Value :    out Item);
         entry Wait (Val : in Item);
         --  Blocks until the protected item's value equals @Val@.
      private
         Value  : Item    := Default;
         Change : Boolean := False;

         entry Wait_Until (Boolean) (Val : in Item);
         Active_Queue : Boolean := False;

      end Protected_Item;

   end Protect_With_Wait;

   ----------------------------------------------------------------------------

end Util.Tasking;
