-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2001, 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <VERSION ID="1.1">
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--    I/O operations on unbounded strings.
--  </PURPOSE>
--
--  <NOT_TASK_SAFE>
--
--  <STORAGE>
--    Dynamic storage allocation in the default pool.
--  </STORAGE>
--
--  <HISTORY>
--    07-JUN-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

with Ada.Text_IO;

package Util.Text.IO is

   pragma Elaborate_Body;

   procedure Put
     (File : in Ada.Text_IO.File_Type;
      Str  : in Unbounded_String);

   procedure Put
     (Str : in Unbounded_String);

   procedure Put_Line
     (File : in Ada.Text_IO.File_Type;
      Str  : in Unbounded_String);

   procedure Put_Line
     (Str : in Unbounded_String);

   procedure Put
     (File : in Ada.Text_IO.File_Type;
      Str  : in Unbounded_String;
      Low  : in Positive;
      High : in Natural);

   procedure Put
     (Str  : in Unbounded_String;
      Low  : in Positive;
      High : in Natural);

   procedure Put_Line
     (File : in Ada.Text_IO.File_Type;
      Str  : in Unbounded_String;
      Low  : in Positive;
      High : in Natural);

   procedure Put_Line
     (Str  : in Unbounded_String;
      Low  : in Positive;
      High : in Natural);

   procedure Get_Line
     (File : in     Ada.Text_IO.File_Type;
      Str  : in out Unbounded_String);

   procedure Get_Line
     (Str : in out Unbounded_String);

   function Get_Line
     (File : in Ada.Text_IO.File_Type)
     return Unbounded_String;

   function Get_Line
     return Unbounded_String;

end Util.Text.IO;
