-------------------------------------------------------------------------------
--
--  <STRONG>Copyright &copy; 2001, 2002 by Thomas Wolf.</STRONG>
--  <BLOCKQUOTE>
--    This piece of software is free software; you can redistribute it and/or
--    modify it under the terms of the  GNU General Public License as published
--    by the Free Software  Foundation; either version 2, or (at your option)
--    any later version. This software is distributed in the hope that it will
--    be useful, but <EM>without any warranty</EM>; without even the implied
--    warranty of <EM>merchantability or fitness for a particular purpose.</EM>
--    See the GNU General Public License for  more details. You should have
--    received a copy of the GNU General Public License with this distribution,
--    see file "<A HREF="GPL.txt">GPL.txt</A>". If not, write to the Free
--    Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
--    USA.
--  </BLOCKQUOTE>
--  <BLOCKQUOTE>
--    As a special exception from the GPL, if other files instantiate generics
--    from this unit, or you link this unit with other files to produce an
--    executable, this unit does not by itself cause the resulting executable
--    to be covered by the GPL. This exception does not however invalidate any
--    other reasons why the executable file might be covered by the GPL.
--  </BLOCKQUOTE>
--
--  <AUTHOR>
--    Thomas Wolf  (TW) <E_MAIL>
--  </AUTHOR>
--
--  <PURPOSE>
--   Stream operations on slices of unbounded strings.
--  </PURPOSE>
--
--  <NOT_TASK_SAFE>
--
--  <NO_STORAGE>
--
--  <HISTORY>
--    11-JUN-2002   TW  Initial version.
--  </HISTORY>
-------------------------------------------------------------------------------

package body Util.Text.Streams is

   procedure Put
     (Stream : access Ada.Streams.Root_Stream_Type'Class;
      Source : in     Unbounded_String)
   is
   begin
      String'Write (Stream, Source.Data.all);
   end Put;

   procedure Put
     (Stream : access Ada.Streams.Root_Stream_Type'Class;
      Source : in     Unbounded_String;
      Low    : in     Positive;
      High   : in     Natural)
   is
   begin
      if Low > Source.Data'Last or else High > Source.Data'Last then
         raise Ada.Strings.Index_Error;
      end if;
      String'Write (Stream, Source.Data (Low .. High));
   end Put;

   procedure Write
     (Stream : access Ada.Streams.Root_Stream_Type'Class;
      Source : in     Unbounded_String;
      Low    : in     Positive;
      High   : in     Natural)
   is
   begin
      if Low > Source.Data'Last or else High > Source.Data'Last then
         raise Ada.Strings.Index_Error;
      end if;
      if Low <= High then
         Natural'Write (Stream, High - Low + 1);
         String'Write  (Stream, Source.Data (Low .. High));
      else
         Natural'Write (Stream, 0);
      end if;
   end Write;

end Util.Text.Streams;
